@@ -79,7 +79,7 @@ void write_bookmarks() {
 	foreachback(bm, bookmarks) {
 		unsigned char *p = stracpy(bm->title);
 		int i;
-		for (i = strlen(p) - 1; i >= 0; i--) if (p[i] < ' '|| p[i] == '|') p[i] = '*';
+		for (i = strlen(p) - 1; i >= 0; i--) if (p[i] < ' '|| p[i] == '|') p[i] = ' ';
 		fputs(p,out);
 		fputc('|', out);
 		fputs(bm->url,out);
