@@ -203,8 +203,6 @@ void ses_forward(struct session *ses)
 	if (!list_empty(ses->history)) {
 		struct frame *frm;
 		l = cur_loc(ses);
-		foreach(frm, l->frames) frm->vs.f = NULL;
-		l->vs.f = NULL;
 	}
 	if (ses->search_word) mem_free(ses->search_word), ses->search_word = NULL;
 	x:
@@ -507,7 +505,8 @@ void download_data(struct status *stat, struct download *down)
 	if (ce->redirect && down->redirect_cnt++ < MAX_REDIRECTS) {
 		unsigned char *u, *p;
 		if (stat->state >= 0) change_connection(&down->stat, NULL, PRI_CANCEL);
-		u = stracpy(ce->redirect);
+		u = join_urls(down->url, ce->redirect);
+		if (!u) goto x;
 		if (!http_bugs.bug_302_redirect) if (!ce->redirect_get && (p = strchr(down->url, POST_CHAR))) add_to_strn(&u, p);
 		mem_free(down->url);
 		down->url = u;
@@ -528,6 +527,7 @@ void download_data(struct status *stat, struct download *down)
 			msg_box(get_download_ses(down)->term, getml(msg, NULL), TEXT(T_WARNING), AL_CENTER | AL_EXTD_TEXT, TEXT(T_DO_YOU_WANT_TO_FOLLOW_REDIRECT_AND_POST_FORM_DATA_TO_URL), "", msg, "?", NULL, down, 3, TEXT(T_YES), down_post_yes, B_ENTER, TEXT(T_NO), down_post_no, 0, TEXT(T_CANCEL), down_post_cancel, B_ESC);
 		}*/
 	}
+	x:
 	foreach(frag, ce->frag) if (frag->offset <= down->last_pos && frag->offset + frag->length > down->last_pos) {
 		int w;
 #ifdef HAVE_OPEN_PREALLOC
@@ -985,16 +985,18 @@ void request_frame(struct session *ses, unsigned char *name, unsigned char *uurl
 		return;
 	}
 	foreach(frm, loc->frames) if (!strcasecmp(frm->name, name)) {
-		url = stracpy(frm->vs.url);
-		if (frm->vs.f && frm->vs.f->f_data && frm->vs.f->f_data->frame) {
-			/*del_from_list(frm);*/
-			request_frameset(ses, frm->vs.f->f_data->frame_desc);
-			/*destroy_vs(&frm->vs);
-			mem_free(frm->name);
-			mem_free(frm);*/
-			mem_free(url);
+		struct f_data_c *fd;
+		foreach(fd, ses->scrn_frames) if (fd->vs == &frm->vs && fd->f_data->frame_desc) {
+			request_frameset(ses, fd->f_data->frame_desc);
 			return;
 		}
+		url = stracpy(frm->vs.url);
+		/*
+		if (frm->vs.f && frm->vs.f->f_data && frm->vs.f->f_data->frame) {
+			request_frameset(ses, frm->vs.f->f_data->frame_desc);
+			mem_free(url);
+			return;
+		}*/
 		goto found;
 	}
 	url = stracpy(uurl);
@@ -1097,8 +1099,9 @@ void end_load(struct status *stat, struct session *ses)
 		}
 		if (d == 1) doc_end_load(stat, ses);
 	}
-	if (stat->state < 0 && stat->state != S_OK && d != 2) {
+	if (stat->state < 0 && stat->state != S_OK && d != 2 && d != 1) {
 		print_error_dialog(ses, stat, TEXT(T_ERROR));
+		if (!d) reload(ses, NC_CACHE);
 	}
 	if (stat->state == S_QUESTIONS) check_questions_queue(ses);
 	print_screen_status(ses);
@@ -1538,7 +1541,7 @@ void win_func(struct window *win, struct event *ev, int fw)
 		case EV_INIT:
 			if (!(ses = win->data = create_session(win)) ||
 			    read_session_info(win->term->fdin, ses, (char *)ev->b + sizeof(int), *(int *)ev->b)) {
-				destroy_terminal(win->term);
+				register_bottom_half((void (*)(void *))destroy_terminal, win->term);
 				return;
 			}
 			/*make_request(ses, 0);*/
